# Generated by roxygen2: do not edit by hand

S3method(htmlTable,data.frame)
S3method(htmlTable,default)
S3method(htmlTable,matrix)
S3method(interactiveTable,default)
S3method(interactiveTable,htmlTable)
S3method(knit_print,htmlTable)
S3method(knit_print,interactiveTable)
S3method(print,htmlTable)
S3method(print,interactiveTable)
S3method(tidyHtmlTable,data.frame)
S3method(tidyHtmlTable,default)
S3method(txtRound,data.frame)
S3method(txtRound,default)
S3method(txtRound,matrix)
S3method(txtRound,table)
export(addHtmlTableStyle)
export(concatHtmlTables)
export(getHtmlTableStyle)
export(getHtmlTableTheme)
export(hasHtmlTableStyle)
export(highlightRow)
export(htmlTable)
export(htmlTableWidget)
export(htmlTableWidgetOutput)
export(interactiveTable)
export(outputInt)
export(prepGroupCounts)
export(pvalueFormatter)
export(renderHtmlTableWidget)
export(setHtmlTableTheme)
export(splitLines4Table)
export(tblNoLast)
export(tblNoNext)
export(tidyHtmlTable)
export(txtInt)
export(txtMergeLines)
export(txtPval)
export(txtRound)
export(vector2string)
import(checkmate)
import(htmlwidgets)
import(magrittr)
importFrom(grDevices,col2rgb)
importFrom(grDevices,colorRampPalette)
importFrom(htmltools,htmlEscape)
importFrom(knitr,asis_output)
importFrom(knitr,knit_print)
importFrom(methods,formalArgs)
importFrom(methods,setClass)
importFrom(rstudioapi,getActiveDocumentContext)
importFrom(rstudioapi,isAvailable)
importFrom(stats,na.omit)
importFrom(stringr,str_interp)
importFrom(stringr,str_replace)
importFrom(stringr,str_replace_all)
importFrom(stringr,str_split)
importFrom(stringr,str_trim)
importFrom(utils,as.roman)
importFrom(utils,browseURL)
importFrom(utils,head)
importFrom(utils,tail)
