#' Average age in Sweden
#'
#' For the vignettes there is a dataset downloaded by using the
#' `get_pxweb_data()` call. The data is from
#' SCB ([Statistics Sweden](https://scb.se/)) and downloaded
#' using the [pxweb package](https://github.com/rOpenGov/pxweb):
#'
#' @example inst/examples/data-SCB_example.R
#'
#' @name SCB
#' @docType data
#' @author Max Gordon \email{max@@gforge.se}
#' @references <https://scb.se>
#' @keywords data
NULL
