---
title: "Pandoc test"
output:
  html_document
---

```{r}
library(htmlTable)
mx <- matrix(1, ncol=1)
colnames(mx) <- c("A")
rownames(mx) <- letters[1]
htmlTable(mx)
```


```{r}
mx[1] <- "Lorem ipsum dolor sit amet, consectetur adipiscing elit, sed do eiusmod tempor incididunt ut labore et dolore magna aliqua. Ut enim ad minim veniam, quis nostrud exercitation ullamco laboris nisi ut aliquip ex ea commodo consequat. Duis aute irure dolor in reprehenderit in voluptate velit esse cillum dolore eu fugiat nulla pariatur. Excepteur sint occaecat cupidatat non proident, sunt in culpa qui officia deserunt mollit anim id est laborum"
colnames(mx) <- c("A")
rownames(mx) <- letters[1]
interactiveTable(mx)
```


```{r}
mx <- matrix(rep(mx, 6), ncol = 2)
interactiveTable(mx)

```

