% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/deprecated.R
\name{pvalueFormatter}
\alias{pvalueFormatter}
\title{Deprecated use \code{\link[=txtPval]{txtPval()}} instead}
\usage{
pvalueFormatter(...)
}
\arguments{
\item{...}{Currently only used for generating warnings of deprecated call}
}
\description{
Deprecated use \code{\link[=txtPval]{txtPval()}} instead
}
\examples{
\dontrun{
# Deprecated function
pvalueFormatter(c(0.10234,0.010234, 0.0010234, 0.000010234))
}
}
\keyword{internal}
